@@ -269,8 +269,12 @@ class GCMT:
             'day': only dayside (defined around 0,0)
             'morning': morning terminator (average around lon=[-100,-80])
             'evening': evening terminator (average around lon=[80,100])
-            Alternatively you may specify a dict in the following way:
-            part = {'lon': [-100,-80], 'lat':[-90,90]} (example for morn. term.)
+            Alternatively you may specify a dict in the following way
+            (example for morn. term.):
+            part = {'lon': [-100,-80], 'lat':[-90,90], 'inv':False}
+            The 'lon', 'lat' specify regions of lon and lat that should be used,
+            whereas 'inv' (optional, default False) gives the option to invert
+            the lon and lat regions (e.g., exclude instead of include for average)
         area_key: str, optional
             Variable key in the dataset for the area of grid cells
         tag : str, optional
@@ -319,8 +323,12 @@ class GCMT:
             'day': only dayside (defined around 0,0)
             'morning': morning terminator (average around lon=[-100,-80])
             'evening': evening terminator (average around lon=[80,100])
-            Alternatively you may specify a dict in the following way:
-            part = {'lon': [-100,-80], 'lat':[-90,90]} (example for morn. term.)
+            Alternatively you may specify a dict in the following way
+            (example for morn. term.):
+            part = {'lon': [-100,-80], 'lat':[-90,90], 'inv':False}
+            The 'lon', 'lat' specify regions of lon and lat that should be used,
+            whereas 'inv' (optional, default False) gives the option to invert
+            the lon and lat regions (e.g., exclude instead of include for average)
         area_key: str, optional
             Variable key in the dataset for the area of grid cells
         temp_key: str, optional
@@ -460,10 +468,13 @@ class GCMT:
         self, v_data="V", var_key_out=None, tag=None
     ):
         """
-        Calculate meridional overturning streamfunction.
-
-        See e.g. Carone et al. (2017), Eq. 7
-
+        Calculate meridional overturning streamfunction. This quantity psi is
+        computed by integrating the zonal-mean meridional velocity \bar V along
+        pressure, and weighting with 2*pi*R_p / g times the cosine of latitude,
+        where R_p is the planetary radius and g is the surface gravity:
+        bar{psi} = 2 pi R_p / g cos{lat} int{bar{V} dp'}
+        (see e.g. Carone et al. (2018), Eq. 7)
+    
         Parameters
         ----------
         v_data: str
@@ -488,6 +499,45 @@ class GCMT:
             dsi, v_data=v_data, var_key_out=var_key_out
         )
 
+    def extend_upward(self, p_low, method=None, n_p=20, T_therm=None,
+                      p_therm_high=None, tag=None):
+        """
+        Extend temperatures of the tagged dataset upward to the given pressure value.
+        Multiple methods can be used to extend the data upwards:
+            - 'isothermal' copies the temperature at lowest pressure and adds layers
+              of the same temperature on top
+            - 'thermosphere' imposes a simple parametrized thermosphere with a hot
+              day-side and gradual decrease to an isothermal night-side
+              (cfr. Baeyens+2022, Section 4.2)
+        Parameters
+        ----------
+        p_low: float or list
+            If a float is given, extend to the given low-pressure limit using an
+            arbitrarily chosen pressure-spacing. If a list is given, use these as
+            pressures to compute temperature extension.
+        method: string
+            Method by which the temperature is to be extended.
+        n_p: int, optional
+            Number of pressure layers that should be added (only used in case p_low
+            is a single number).
+        T_therm: float, optional
+            Maximal day-side temperature for the upper thermosphere (required for
+            the 'thermosphere' method.)
+        p_therm_high: float, optional
+            The high-pressure limit at which thermospheric heating should start, in
+            native pressure units (required for the 'thermosphere' method).
+        tag : str, optional
+            The tag of the dataset that should be used. If no tag is provided,
+            and multiple datasets are available, an error is raised.
+        Returns
+        -------
+        temp_array_ext: DataArray
+            The extended temperature array.
+        """
+        dsi = self.get_one_model(tag)
+        return mani.m_extend_upward(dsi.T, p_low, method=method, n_p=n_p,
+                                    T_therm=T_therm, p_therm_high=p_therm_high)
+
     # ======================================================
     #   Reading and writing functions
     # ======================================================
