@@ -101,6 +101,16 @@ def test_prt_interface(petitradtrans_testdata, all_raw_testdata):
             normalize=True,
         )
 
+    # test transit calculation set up
+    interface.set_data(time=expected["times"][-1], terminator_avg=True)
+    assert (interface.dsi['lon'].values == [-90, 90]).all()
+    assert interface.dsi['lat'].values == [0]
+
+    # test transit calculation
+    interface.chem_from_poorman("T", co_ratio=0.55, feh_ratio=0.0)
+    wave, spectra = interface.calc_transit_spectrum(mmw=2.33)
+    assert sum(wave) == 28.373223101833855
+
     # Test if Pa works
     interface.dsi.attrs["p_unit"] = "Pa"
     interface.set_data(time=expected["times"][-1])
@@ -202,3 +212,27 @@ def test_general_interface(all_nc_testdata, petitradtrans_testdata):
 
     with pytest.raises(ValueError):
         interface.chemistry.to_prt(["wrong_species"], np.array(0.5))
+
+
+def test_PAC_interface(all_nc_testdata):
+    """General test for PAC interface class."""
+    dirname, expected = all_nc_testdata
+
+    tools = GCMT(p_unit="bar", time_unit="day")  # create a GCMT object
+    tools.read_reduced(data_path=dirname)
+
+    from gcm_toolkit.utils.interface import PACInterface
+
+    pac1d = PACInterface(tools, 1)
+    pac2d = PACInterface(tools, 2)
+    with pytest.raises(ValueError):
+        PACInterface(tools, 3)
+
+    pac1d.set_data(time=expected["times"][-1])
+    pac2d.set_data(time=expected["times"][-1])
+
+    with pytest.raises(OSError):
+        pac1d.write_inputfile('non/existent/path')
+        pac2d.write_inputfile('non/existent/path')
+        pac2d.generate_lptfile('non/existent/path')
+        pac2d.generate_aptfiles('non/existent/path')
