@@ -36,8 +36,12 @@ def m_add_horizontal_average(
         'day': only dayside (defined around 0,0)
         'morning': morning terminator (average around lon=[-100,-80])
         'evening': evening terminator (average around lon=[80,100])
-        Alternatively you may specify a dict in the following way:
-        part = {'lon': [-100,-80], 'lat':[-90,90]} (example for morn. term.)
+        Alternatively you may specify a dict in the following way
+        (example for morn. term.):
+        part = {'lon': [-100,-80], 'lat':[-90,90], 'inv':False}
+        The 'lon', 'lat' specify regions of lon and lat that should be used,
+        whereas 'inv' (optional, default False) gives the option to invert
+        the lon and lat regions (e.g., exclude instead of include for average)
     area_key: str, optional
         Variable key in the dataset for the area of grid cells
 
@@ -66,50 +70,55 @@ def m_add_horizontal_average(
 
     # Determine the area over which we want to average:
     if isinstance(part, str):
-        if part == "day":
-            area = xr.where(abs(dsi[c["lon"]]) < 90.0, dsi[area_key], 0)
+        if part == "global":
+            part_internal = {}
+            wrt.write_status("INFO", "Performing global average")
+        elif part == "day":
+            part_internal = {"lon": [-90, 90]}
             wrt.write_status("INFO", "Performing dayside average")
         elif part == "night":
-            area = xr.where(abs(dsi[c["lon"]]) < 90.0, 0, dsi[area_key])
+            part_internal = {"lon": [-90, 90], "inv": True}
             wrt.write_status("INFO", "Performing nightside average")
         elif part == "evening":
-            area = xr.where(
-                np.logical_and(dsi[c["lon"]] > 80, dsi[c["lon"]] < 100),
-                dsi[area_key],
-                0,
-            )
+            part_internal = {"lon": [80, 100]}
             wrt.write_status("INFO", "Performing evening terminator average")
         elif part == "morning":
-            area = xr.where(
-                np.logical_and(dsi[c["lon"]] > -100, dsi[c["lon"]] < -80),
-                dsi[area_key],
-                0,
-            )
+            part_internal = {"lon": [-100, -80]}
             wrt.write_status("INFO", "Performing morning terminator average")
-        elif part == "global":
-            wrt.write_status("INFO", "Performing global average")
-            area = dsi[area_key]
         else:
             raise ValueError(
                 "If you specify a string for part, it needs to be either"
                 " morning, evening, day, night or global"
             )
-
     elif isinstance(part, dict):
-        lon_bool = np.logical_and(
-            dsi[c["lon"]] <= max(part["lon"]),
-            dsi[c["lon"]] >= min(part["lon"]),
-        )
-        lat_bool = np.logical_and(
-            dsi[c["lat"]] <= max(part["lat"]),
-            dsi[c["lat"]] >= min(part["lat"]),
-        )
-        combined_bool = np.logical_and(lat_bool, lon_bool)
-        area = xr.where(combined_bool, dsi[area_key], 0)
-
+        wrt.write_status("INFO", "Performing user specified average")
+        part_internal = part
     else:
         raise ValueError("Please use a dict or a string for part.")
 
+    if lon := part_internal.get("lon"):
+        lon_bool = np.logical_and(
+            dsi[c["lon"]] <= max(lon),
+            dsi[c["lon"]] >= min(lon),
+        )
+    else:
+        lon_bool = xr.ones_like(dsi[area_key], dtype=bool)
+
+    if lat := part_internal.get("lat"):
+        lat_bool = np.logical_and(
+            dsi[c["lat"]] <= max(lat),
+            dsi[c["lat"]] >= min(lat),
+        )
+    else:
+        lat_bool = xr.ones_like(dsi[area_key], dtype=bool)
+
+    combined_bool = np.logical_and(lat_bool, lon_bool)
+
+    if part_internal.get("inv", False):
+        area = xr.where(combined_bool, 0, dsi[area_key])
+    else:
+        area = xr.where(combined_bool, dsi[area_key], 0)
+
     avg = (area * data).sum(dim=[c["lon"], c["lat"]]) / area.sum(
         dim=[c["lon"], c["lat"]]
     )
@@ -335,8 +344,12 @@ def m_add_rcb(
         'day': only dayside (defined around 0,0)
         'morning': morning terminator (average around lon=[-100,-80])
         'evening': evening terminator (average around lon=[80,100])
-        Alternatively you may specify a dict in the following way:
-        part = {'lon': [-100,-80], 'lat':[-90,90]} (example for morn. term.)
+        Alternatively you may specify a dict in the following way
+        (example for morn. term.):
+        part = {'lon': [-100,-80], 'lat':[-90,90], 'inv':False}
+        The 'lon', 'lat' specify regions of lon and lat that should be used,
+        whereas 'inv' (optional, default False) gives the option to invert
+        the lon and lat regions (e.g., exclude instead of include for average)
     area_key: str, optional
         Variable key in the dataset for the area of grid cells
     temp_key: str, optional
@@ -423,3 +436,137 @@ def m_add_meridional_overturning(dsi, v_data="V", var_key_out=None):
         dsi.update({var_key_out: psi})
 
     return psi
+
+def m_extend_upward(temp_array, p_low, method=None, n_p=20, T_therm=None,
+                    p_therm_high=None):
+    """
+    Extend the given temperature DataArray upward to the given pressure value.
+    Multiple methods can be used to extend the data upwards:
+        - 'isothermal' copies the temperature at lowest pressure and adds layers
+          of the same temperature on top
+        - 'thermosphere' imposes a simple parametrized thermosphere with a hot
+          day-side and gradual decrease to an isothermal night-side
+          (cfr. Baeyens+2022, Section 4.2)
+
+    Parameters
+    ----------
+    temp_array: DataArray
+        The temperature data-array that is to be extended to lower pressures.
+    p_low: float or list
+        If a float is given, extend to the given low-pressure limit using an
+        arbitrarily chosen pressure-spacing. If a list is given, use these as
+        pressures to compute temperature extension.
+    method: string
+        Method by which the temperature is to be extended.
+    n_p: int, optional
+        Number of pressure layers that should be added (only used in case p_low
+        is a single number).
+    T_therm: float, optional
+        Maximal day-side temperature for the upper thermosphere (required for
+        the 'thermosphere' method.)
+    p_therm_high: float, optional
+        The high-pressure limit at which thermospheric heating should start, in
+        native pressure units (required for the 'thermosphere' method).
+
+    Returns
+    -------
+    temp_array_ext: DataArray
+        The extended temperature array.
+    """
+    # Check if the input pressure is suitable
+    p_low_current = min(temp_array.Z.values) # lowest pressure in current data
+    if hasattr(p_low, "__len__"):           # p_low is a list
+        p_low = [ip for ip in p_low if ip < p_low_current]
+        p_low = sorted(p_low)[::-1]
+    elif isinstance(p_low, (int, float)):   # p_low is a scalar
+        p_low = np.logspace(np.log10(p_low_current), np.log10(p_low), n_p+1)
+        p_low = p_low[1:]
+    else:                                   # p_low is something else
+        wrt.write_status('ERROR', 'p_low should be a float, int, list, or array.')
+
+    # Now p_low consists of a sorted list of low pressures
+
+    # Construct an extended dataArray (initially filled with zeros)
+    new_coords = dict(time= temp_array.time, Z=p_low, lat=temp_array.lat,
+                      lon=temp_array.lon)
+    # NB: in order to avoid problems with merging the extended and the original
+    # temperature array, we need to copy over all coordinates. We just assign
+    # them NaN values since we aren't interested in their values.
+    for c in temp_array.coords:
+        if c not in new_coords:
+            new_coords[c] = np.nan
+    nz = len(new_coords['Z'])
+    nlat = len(new_coords['lat'])
+    nlon = len(new_coords['lon'])
+    if 'time' in temp_array.dims:
+        nt = len(new_coords['time'])
+        time_coord = True # flag for an explicit time coordinate --> 4D array
+        extended_values = np.zeros((nt, nz, nlat, nlon))
+    else:
+        nt = 1
+        time_coord = False # no explicit time coordinate --> 3D array
+        extended_values = np.zeros((nz, nlat, nlon))
+
+    # Fill the extended data according to the chosen method
+    # ISOTHERMAL
+    if method == 'isothermal':
+        iso_slice = temp_array.isel(Z=-1).values    # constant upper temperature
+        for i in range(0, nz):
+            if time_coord:
+                extended_values[:,i,:,:] = iso_slice # copy for each new pressure
+            else:
+                extended_values[i,:,:] = iso_slice # copy for each new pressure
+
+    # THERMOSPHERE
+    elif method == 'thermosphere':
+        import math
+        # Check if all parameters are given
+        if T_therm is None:
+            wrt.write_status('ERROR', 'Argument "T_therm" is required for the thermosphere method.')
+        if p_therm_high is None:
+            wrt.write_status('ERROR', 'Argument "p_therm_high" is required for the thermosphere method.')
+        p_therm_low = min(p_low)   # minimum pressure for thermosphere extention
+
+        # For each time step...
+        for it in range(0, nt):
+            # and each vertical column...
+            for ilon in range(0, nlon):
+                for ilat in range(0, nlat):
+                    # ... save the top temperature as minimum
+                    if time_coord:
+                        T_min = temp_array.isel(time=it, Z=-1, lat=ilat, lon=ilon)
+                    else:
+                        T_min = temp_array.isel(Z=-1, lat=ilat, lon=ilon)
+                    # ... calculate angle of incidence
+                    mu = math.cos(math.radians(T_min.lon)) * \
+                         abs( math.cos(math.radians(T_min.lat)) )
+                    # ... determine the 'thermospheric' upper temperature.
+                    T_max = max(T_min, T_therm * mu)
+
+                    # For each pressure level...
+                    for i in range(0, nz):
+                        ip = p_low[i]
+                        # ... if the pressure is below the thermosphere...
+                        if ip < p_therm_high and mu >= 0:
+                            # ... do the interpolation to T_max
+                            T_new = T_min + (T_max-T_min)/(math.log10(p_therm_high)-math.log10(p_therm_low)) * (math.log10(p_therm_high)-math.log10(ip))
+                        else: # ... otherwise don't change anything.
+                            T_new = T_min
+                        if time_coord:
+                            extended_values[it, i, ilat, ilon] = T_new
+                        else:
+                            extended_values[i, ilat, ilon] = T_new
+    else:
+        msg = 'Specify either "isothermal" or "thermosphere" as method.'
+        wrt.write_status('ERROR', msg)
+
+    # Make the new extended data into a DataArray
+    extension = xr.DataArray(extended_values,
+                             coords=new_coords,
+                             dims=temp_array.dims)
+
+    # Combine the extended and original data
+    temp_array = xr.concat([temp_array, extension], dim='Z',
+                           compat='no_conflicts')
+
+    return temp_array
